@@ -1,12 +1,11 @@
-// Copyright 2019 Google LLC
-//
+// Copyright 2019 The age Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style
-// license that can be found in the LICENSE file or at
-// https://developers.google.com/open-source/licenses/bsd
+// license that can be found in the LICENSE file.
 
 package agessh
 
 import (
+	"crypto"
 	"crypto/ed25519"
 	"crypto/rsa"
 	"fmt"
@@ -107,19 +106,20 @@ func (i *EncryptedSSHIdentity) Unwrap(stanzas []*age.Stanza) (fileKey []byte, er
 		return nil, fmt.Errorf("failed to decrypt SSH key file: %v", err)
 	}
 
+	var pubKey interface {
+		Equal(x crypto.PublicKey) bool
+	}
 	switch k := k.(type) {
 	case *ed25519.PrivateKey:
 		i.decrypted, err = NewEd25519Identity(*k)
-		// TODO: here and below, better check that the two public keys match,
-		// rather than just the type.
-		if i.pubKey.Type() != ssh.KeyAlgoED25519 {
-			return nil, fmt.Errorf("mismatched private (%s) and public (%s) SSH key types", ssh.KeyAlgoED25519, i.pubKey.Type())
-		}
+		pubKey = k.Public().(ed25519.PublicKey)
+	// ParseRawPrivateKey returns inconsistent types. See Issue 429.
+	case ed25519.PrivateKey:
+		i.decrypted, err = NewEd25519Identity(k)
+		pubKey = k.Public().(ed25519.PublicKey)
 	case *rsa.PrivateKey:
 		i.decrypted, err = NewRSAIdentity(k)
-		if i.pubKey.Type() != ssh.KeyAlgoRSA {
-			return nil, fmt.Errorf("mismatched private (%s) and public (%s) SSH key types", ssh.KeyAlgoRSA, i.pubKey.Type())
-		}
+		pubKey = &k.PublicKey
 	default:
 		return nil, fmt.Errorf("unexpected SSH key type: %T", k)
 	}
@@ -127,5 +127,9 @@ func (i *EncryptedSSHIdentity) Unwrap(stanzas []*age.Stanza) (fileKey []byte, er
 		return nil, fmt.Errorf("invalid SSH key: %v", err)
 	}
 
+	if exp := i.pubKey.(ssh.CryptoPublicKey).CryptoPublicKey(); !pubKey.Equal(exp) {
+		return nil, fmt.Errorf("mismatched private and public SSH key")
+	}
+
 	return i.decrypted.Unwrap(stanzas)
 }
