@@ -1,8 +1,6 @@
-// Copyright 2019 Google LLC
-//
+// Copyright 2019 The age Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style
-// license that can be found in the LICENSE file or at
-// https://developers.google.com/open-source/licenses/bsd
+// license that can be found in the LICENSE file.
 
 package main
 
@@ -10,13 +8,13 @@ import (
 	"bytes"
 	"errors"
 	"fmt"
-	"os"
-	"runtime"
 
 	"filippo.io/age"
-	"golang.org/x/term"
 )
 
+// LazyScryptIdentity is an age.Identity that requests a passphrase only if it
+// encounters an scrypt stanza. After obtaining a passphrase, it delegates to
+// ScryptIdentity.
 type LazyScryptIdentity struct {
 	Passphrase func() (string, error)
 }
@@ -101,44 +99,6 @@ func (i *EncryptedIdentity) decrypt() error {
 	if err != nil {
 		return fmt.Errorf("failed to decrypt identity file: %v", err)
 	}
-	i.identities, err = age.ParseIdentities(d)
+	i.identities, err = parseIdentities(d)
 	return err
 }
-
-// readPassphrase reads a passphrase from the terminal. It does not read from a
-// non-terminal stdin, so it does not check stdinInUse.
-func readPassphrase(prompt string) ([]byte, error) {
-	var in, out *os.File
-	if runtime.GOOS == "windows" {
-		var err error
-		in, err = os.OpenFile("CONIN$", os.O_RDWR, 0)
-		if err != nil {
-			return nil, err
-		}
-		defer in.Close()
-		out, err = os.OpenFile("CONOUT$", os.O_WRONLY, 0)
-		if err != nil {
-			return nil, err
-		}
-		defer out.Close()
-	} else if _, err := os.Stat("/dev/tty"); err == nil {
-		tty, err := os.OpenFile("/dev/tty", os.O_RDWR, 0)
-		if err != nil {
-			return nil, err
-		}
-		defer tty.Close()
-		in, out = tty, tty
-	} else {
-		if !term.IsTerminal(int(os.Stdin.Fd())) {
-			return nil, fmt.Errorf("standard input is not a terminal, and /dev/tty is not available: %v", err)
-		}
-		in, out = os.Stdin, os.Stderr
-	}
-	fmt.Fprintf(out, "%s ", prompt)
-	// Use CRLF to work around an apparent bug in WSL2's handling of CONOUT$.
-	// Only when running a Windows binary from WSL2, the cursor would not go
-	// back to the start of the line with a simple LF. Honestly, it's impressive
-	// CONIN$ and CONOUT$ even work at all inside WSL2.
-	defer fmt.Fprintf(out, "\r\n")
-	return term.ReadPassword(int(in.Fd()))
-}
